/**
/// HashTag.Core Library
/// Copyright © 2010-2013
///
/// This module is Copyright © 2010-2013 Steve Powell
/// All rights reserved.
///
/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the Microsoft Public License (Ms-PL)
/// 
/// This library is distributed in the hope that it will be
/// useful, but WITHOUT ANY WARRANTY; without even the implied
/// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
/// PURPOSE.  See theMicrosoft Public License (Ms-PL) License for more
/// details.
///
/// You should have received a copy of the Microsoft Public License (Ms-PL)
/// License along with this library; if not you may 
/// find it here: http://www.opensource.org/licenses/ms-pl.html
///
/// Steve Powell, hashtagdonet@gmail.com
**/
using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Configuration;
using System.IO;

using System.Threading;
using HashTag.Text;
using HashTag.Diagnostics.Logging;
using System.Web;

namespace HashTag.Diagnostics
{
    /// <summary>
    /// Base class for writing messages to persistant store.  By default creates log file in application's executing folder
    /// </summary>
    public sealed partial class Log: IDisposable 
    {
        

        private LogConfiguration _config;
        
        private static ReaderWriterLockSlim _lastChanceLock = new ReaderWriterLockSlim(LockRecursionPolicy.SupportsRecursion);

        private static string _defaultKey = string.Empty;
        private string _applicationKey = string.Empty;

        private LoggerLevels _logLevels;

        private LogMessageBuilder makeMessageBuilder(MessageLevel level, string logName)
        {
            LogMessage msg = new LogMessage();
            msg.LoggerName = logName;
            msg.Severity = level;
            msg.OnWrite += Log_OnWrite;
            switch(level)
            {
                case MessageLevel.Critical:
                case MessageLevel.Error:
                case MessageLevel.Warning:
                    if (HttpContext.Current != null)
                    {
                        msg.RuntimeContext = new RuntimeContext(RuntimeContextOptions.MachineContext);
                    }
                    else
                    {
                        msg.RuntimeContext = new RuntimeContext(RuntimeContextOptions.MachineContext | RuntimeContextOptions.HttpRequestContext);
                    }
                    break;
            }
            return new LogMessageBuilder(msg);
        }

        public LogMessageBuilder Critical
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Critical, LogName);

            }
        }

        public LogMessageBuilder Error
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Error, LogName);
            }
        }

        public LogMessageBuilder Warning
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Warning, LogName);

            }
        }

        public LogMessageBuilder Info
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Information, LogName);
            }
        }

        public LogMessageBuilder Stop
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Stop, LogName);
            }
        }

        public LogMessageBuilder Debug
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Debug, LogName);
            }
        }

        public LogMessageBuilder Start
        {
            get
            {
                return makeMessageBuilder(MessageLevel.Start, LogName);
            }
        }


        /// <summary>
        /// Determines if a particular LogLevel (serverity of a single message) is enabled to be logged 
        /// </summary>
        /// <param name="level">Message log level to be checked</param>
        /// <returns>True if level is enabled to be logged or false if message cannot be logged in this instance</returns>
        /// <exception cref="ArgumentException">Thrown when level doesn't fall within a known set of elements</exception>
        public bool IsLevelEnabled(MessageLevel level)
        {
            switch (level)
            {
                case MessageLevel.Debug: return ((_logLevels & LoggerLevels.Debug) == LoggerLevels.Debug) && (OnLogDebugEventHandler != null);
                case MessageLevel.Start: return ((_logLevels & LoggerLevels.Start) == LoggerLevels.Start) && (OnLogStartEventHandler != null);
                case MessageLevel.Stop: return ((_logLevels & LoggerLevels.Stop) == LoggerLevels.Stop) && (OnLogStopEventHandler != null);
                case MessageLevel.Information: return ((_logLevels & LoggerLevels.Information) == LoggerLevels.Information) && (OnLogInformationEventHandler != null);
                case MessageLevel.Warning: return ((_logLevels & LoggerLevels.Warning) == LoggerLevels.Warning) && (OnLogWarningEventHandler != null);
                case MessageLevel.Error: return ((_logLevels & LoggerLevels.Error) == LoggerLevels.Error) && (OnLogErrorEventHandler != null);
                case MessageLevel.Critical: return ((_logLevels & LoggerLevels.Critical) == LoggerLevels.Critical) && (OnLogCriticalEventHandler != null);
                case MessageLevel.Off: return ((level & MessageLevel.Off) == MessageLevel.Off);
                default:
                    throw new ArgumentException("Unable to identify LogLevel of '{0}'", level.ToString());
            }
        }
        /// <summary>
        /// Sets or clears logging of specific kinds of messages
        /// </summary>
        /// <param name="logLevel">LogLevel that needs to be cleared or set</param>
        /// <param name="value">True to turn on LogLevel.  False to turn off logging at that particular LogLevel</param>
        /// <returns>New active log level</returns>
        public LoggerLevels SetLogLevel(LoggerLevels logLevel, bool value)
        {
            if ((logLevel & LoggerLevels.Start) == LoggerLevels.Start)
            {
                if (value == true) // turn on debug
                {
                    _logLevels |= LoggerLevels.Start;
                    if (OnLogDebugEventHandler == null) OnLogStartEventHandler += Write;
                }
                else // turn off Debug
                {
                    _logLevels &= ~LoggerLevels.Start;
                }
            }
            if ((logLevel & LoggerLevels.Stop) == LoggerLevels.Stop)
            {
                if (value == true) // turn on debug
                {
                    _logLevels |= LoggerLevels.Stop;
                    if (OnLogDebugEventHandler == null) OnLogStartEventHandler += Write;
                }
                else // turn off Debug
                {
                    _logLevels &= ~LoggerLevels.Stop;
                }
            }

            if ((logLevel & LoggerLevels.Debug) == LoggerLevels.Debug)
            {
                if (value == true) // turn on debug
                {
                    _logLevels |= LoggerLevels.Debug;
                    if (OnLogDebugEventHandler == null) OnLogDebugEventHandler += Write;
                }
                else // turn off Debug
                {
                    _logLevels &= ~LoggerLevels.Debug;
                }
            }
        
            if ((logLevel & LoggerLevels.Information) == LoggerLevels.Information)
            {
                if (value == true) // turn on Information
                {
                    _logLevels |= LoggerLevels.Information;
                    if (OnLogInformationEventHandler == null) OnLogInformationEventHandler += Write;
                }
                else // turn off Information
                {
                    _logLevels &= ~LoggerLevels.Information;
                }
            }
            if ((logLevel & LoggerLevels.Warning) == LoggerLevels.Warning)
            {
                if (value == true) // turn on Warning
                {
                    _logLevels |= LoggerLevels.Warning;
                    if (OnLogWarningEventHandler == null) OnLogWarningEventHandler += Write;
                }
                else // turn off Warning
                {
                    _logLevels &= ~LoggerLevels.Warning;
                }
            }
            if ((logLevel & LoggerLevels.Error) == LoggerLevels.Error)
            {
                if (value == true) // turn on Error
                {
                    _logLevels |= LoggerLevels.Error;
                    if (OnLogErrorEventHandler == null) OnLogErrorEventHandler += Write;
                }
                else // turn off Error
                {
                    _logLevels &= ~LoggerLevels.Error;
                }
            }
            
            if ((logLevel & LoggerLevels.Critical) == LoggerLevels.Critical)
            {
                if (value == true) // turn on Critical
                {
                    _logLevels |= LoggerLevels.Critical;
                    if (OnLogCriticalEventHandler == null) OnLogCriticalEventHandler += Write;
                }
                else // turn off Critical
                {
                    _logLevels &= ~LoggerLevels.Critical;
                }
            }


            return _logLevels;
        }


        /// <summary>
        /// Asynchronous buffer that queues up log messages before persisting them to storage
        /// </summary>
        private static MessageBuffer  _logBuffer = new MessageBuffer();

        #region Logging
        /// <summary>
        /// Write message to logging store.  This is the single exit point from HashTag logging into logging provider (TraceSource, NLog, Log4Net, etc)
        /// </summary>
        /// <param name="message">Message to be logged</param>
        public void Write(object message)  //EXTENSION: Since all logging goes through this method, this method is where customization might be most appropriate
        {
            LogMessage msg = message as LogMessage;
            if (msg == null)
            {
                Log.LastChanceLog("Cannot convert '{0}' to type 'LogMessage'.  Only LogMessage objects can be used with this logger",
                    message.GetType().ToString());
                return;
            }
            try
            {
                if (IsLevelEnabled(msg.Severity) == false) return; //this is the final check within the system if this message should be logged

                msg.ActivityId = (msg.ActivityId == Guid.Empty) ? CoreConfig.ActivityId : msg.ActivityId;
                _logBuffer.Submit(msg); //submit message to internal cache and immediately return               
            }
            catch (Exception ex)
            {
                LastChanceLog(MessageLevel.Error, Utils.Expand(ex));
            }
        }


        /// <summary>
        /// Write a message to persistent store
        /// </summary>
        /// <param name="message">Message that will be persisted</param>
        public void Write(LogMessage message)
        {
            if (IsLevelEnabled(message.Severity) == false) return;
            message.LoggerName = this.LogName;
            //_logBuffer.Submit(message);
            Write((object)message);            
        }

        #endregion Logging

        /// <summary>
        /// Close log and flush any buffered log messages
        /// </summary>
        /// <param name="isDisposing">True if being called from explict Dispose().  False if being called from destructor</param>
        public void Dispose(bool isDisposing)
        {
            if (isDisposing == true)
            {

                //TODO clear event handlers here....
                _logBuffer.Dispose();
                _logBuffer = null;
            }
            GC.SuppressFinalize(this);
        }


        /// <summary>
        /// IDisposable implementation
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
        }

        /// <summary>
        /// Destructor
        /// </summary>
        ~Log()
        {
            if (_logBuffer != null)
            {
                _logBuffer.Dispose();
            }
            Dispose(false);
        }
    }
}