/**
/// HashTag.Core Library
/// Copyright © 2010-2013
///
/// This module is Copyright © 2010-2013 Steve Powell
/// All rights reserved.
///
/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the Microsoft Public License (Ms-PL)
/// 
/// This library is distributed in the hope that it will be
/// useful, but WITHOUT ANY WARRANTY; without even the implied
/// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
/// PURPOSE.  See theMicrosoft Public License (Ms-PL) License for more
/// details.
///
/// You should have received a copy of the Microsoft Public License (Ms-PL)
/// License along with this library; if not you may 
/// find it here: http://www.opensource.org/licenses/ms-pl.html
///
/// Steve Powell, hashtagdonet@gmail.com
**/
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HashTag.Diagnostics.Logging;

namespace HashTag.Diagnostics
{
    /// <summary>
    /// Contains global settings for logging subsystem
    /// </summary>
    public class LogConfiguration
    {
        /// <summary>
        /// Default constructor.  Intialized to a well-defined state
        /// </summary>
        public LogConfiguration()
        {
            LastChance = new LastChanceConfig();
            LastChance.MinimumValue = MessageLevel.Error;
            
            AutoWriteOnDispose = true;

            EnforceSLABCompatibility = false;
            HostName = Environment.MachineName;

            AutoHttpContextLevel = MessageLevel.Error;
            HttpCapture = HttpCaptureFlags.Form;         
            AutoCallerInfoLevel = MessageLevel.Off;
            AutoRuntimeContextLevel = MessageLevel.Error;
            
            LoggerLevels = LoggerLevels.Vital;
        }

        /// <summary>
        /// Defines the enabled message levels (default: Vital)
        /// </summary>
        public LoggerLevels LoggerLevels { get; set; }

        /// <summary>
        /// Amount of information to collect from HTTPContext (default: Form)
        /// </summary>
        public HttpCaptureFlags  HttpCapture { get; set; }

        /// <summary>
        /// Automatically call Write if message goes out of scope and it hasn't been written yet (default: true)
        /// </summary>
        public bool AutoWriteOnDispose { get; set; }

        /// <summary>
        /// Machine host name which captured the message
        /// </summary>
        public string HostName { get; set; }
        
        private static string _applicationName;
        private static string _environmentName;

        /// <summary>
        /// Required non-empty value of application running (Application.Key)
        /// </summary>
        public string ApplicationName
        {
            get
            {
                if (string.IsNullOrEmpty(_applicationName))
                {
                    _applicationName = CoreConfig.ApplicationKey;
                }
                return _applicationName;
            }
            set
            {
                _applicationName = value;
            }
        }

        /// <summary>
        /// Required non-empty value of environment where code is running in (e.g. dev, stage, prod, hot fix) (Application.Environment)
        /// </summary>
        public string EnvironmentName
        {
            get
            {
                if (string.IsNullOrEmpty(_environmentName))
                {
                    _environmentName = CoreConfig.ActiveEnvironment;
                }
                return _environmentName;
            }
        }

        /// <summary>
        /// Forces logging runtime to not allow certain field types and values (default:false)
        /// </summary>
        public bool EnforceSLABCompatibility { get; set; }

        /// <summary>
        /// Least severity where runtime will collect HttpContext information (default: Error)
        /// </summary>
        public MessageLevel AutoHttpContextLevel { get; set; }

        /// <summary>
        /// Least severity where runtime will collect stack trace (default: Error)
        /// </summary>
        public MessageLevel AutoCallerInfoLevel { get; set; }

        /// <summary>
        /// Least severity where runtime will collect machine and other environmental information (default: Error)
        /// </summary>
        public MessageLevel AutoRuntimeContextLevel { get; set; }

        /// <summary>
        /// Called by runtime to get a new logging message
        /// </summary>
        public IMessageFactory MessageFactory { get; set; }

        /// <summary>
        /// Configuration for the last chance log (where internal runtime messages will be sent)
        /// </summary>
        public LastChanceConfig LastChance { get; set; }

        /// <summary>
        /// Sets the routine which will persist log messages to permanent storage
        /// </summary>
        public ILogDispatcher  Dispatcher { get; set; }

        /// <summary>
        /// Configuration settings for LastChanceLog
        /// </summary>
        public class LastChanceConfig
        {
            /// <summary>
            /// Code that will persist messages to internal logging store
            /// </summary>
            public ILastChanceLog LCLogger { get; set; }

            /// <summary>
            /// Least level of messages which will be persisted to last chance log
            /// </summary>
            public MessageLevel MinimumValue { get; set; }

            /// <summary>
            /// Path to where last-chance log will be written
            /// </summary>
            public string FilePath { get; set; }
        }
    }

}